@@ -24,6 +24,10 @@
 
 #include "braille.h"
 
+
+void brltty_init ();
+void brltty_terminate ();
+gboolean brltty_check_if_present ();
 gint brltty_brl_open_device (gchar          *device_name, 
 			    const gchar     *port,
 			    BRLDevCallback  device_callback,
