@@ -54,7 +54,7 @@ typedef struct
     gchar 	 *cursor_style;	
     gchar 	 *translation_table;
     
-    gint  	 port;
+    gchar  	 *port;
     gint  	 position_sensor;
     gint  	 optical_sensor;
     gint  	 offset;
@@ -109,7 +109,7 @@ udp_sock_glib_cb(GIOChannel *source,
 {	
     BRLInEvent event;
     BrlPackage package;
-    gint clilen;
+    socklen_t clilen;
     gint n;
 
 /*    if (!remotesockstate) return FALSE; */
@@ -642,7 +642,7 @@ gboolean
 src_braille_restart ()
 {
     gchar *device = g_strdup (src_braille->device);
-    gint port = src_braille->port;
+    gchar *port = g_strdup (src_braille->port);
 
     if (src_use_braille)
 	src_braille_terminate ();
@@ -650,11 +650,9 @@ src_braille_restart ()
 	
     src_braille->device = device;
     src_braille->port = port;	
+
     src_use_braille = src_braille_init ();
 
-    if (src_use_braille_monitor)
-    {
-    }
     return src_use_braille;
 }
 
@@ -759,10 +757,55 @@ src_braille_setting_new ()
     return braille;
 }
 
+static gchar*
+src_braille_get_port_from_no (gint port_no)
+{
+    gchar *port = NULL;
+
+    if (MIN_SERIAL_PORT_NO > port_no ||
+	    MAX_SERIAL_PORT_NO < port_no) 
+    {
+	port_no = DEFAULT_BRAILLE_PORT_NO;
+    	SET_BRAILLE_CONFIG_DATA (BRAILLE_PORT_NO, CFGT_INT, &port_no);
+    }   
+    switch (port_no)
+    {
+	case -1: port = NULL; 	break;
+	case  0: port = g_strdup (TTYNAME0); 	break;
+	case  1: port = g_strdup (TTYNAME1);	break;
+	case  2: port = g_strdup (TTYNAME2); 	break;
+	case  3: port = g_strdup (TTYNAME3);	break;
+	default:
+	    fprintf (stderr, "\nbrl_open_port: Invalid serial port number %d", port_no);
+	    break;
+    }
+    if (!port && DEFAULT_BRAILLE_PORT)
+	port = g_strdup (DEFAULT_BRAILLE_PORT);
+
+    return port;
+}
+
+
+static gchar*
+src_braille_get_port ()
+{
+    gchar *port = DEFAULT_BRAILLE_PORT;
+
+    GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_PORT, CFGT_STRING, 
+					      &port, &port);
+    if (!port || !port[0])
+    {
+	gint port_no = DEFAULT_BRAILLE_PORT_NO;
+        GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_PORT_NO, CFGT_INT, 
+					      &port_no, &port_no);
+	port = src_braille_get_port_from_no (port_no);
+    }
+    return port;
+}
+
 static void
 src_braille_load_values (SRCBraille *braille)
 {
-    gint default_port = DEFAULT_BRAILLE_PORT_NO;
     gchar *default_braille_transaltion_table;
     gchar *default_braille_style;
     gchar *default_braille_cursor_style;
@@ -778,18 +821,8 @@ src_braille_load_values (SRCBraille *braille)
 					      &braille->device, 
 					      default_device);
     }
-        
-    
-    GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_PORT_NO, CFGT_INT, 
-					      &braille->port, 
-					      &default_port);
-    
-    if (MIN_SERIAL_PORT_NO > braille->port || 
-    	    MAX_SERIAL_PORT_NO < braille->port) 
-    {
-	braille->port = DEFAULT_BRAILLE_PORT_NO;
-    	SET_BRAILLE_CONFIG_DATA (BRAILLE_PORT_NO, CFGT_INT, &braille->port);
-    }    
+    g_free (braille->port);
+    braille->port = src_braille_get_port ();
     
     default_braille_transaltion_table = DEFAULT_BRAILLE_TRANSLATION;
     GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_TRANSLATION, CFGT_STRING, 
@@ -843,15 +876,15 @@ src_braille_init ()
     if (!brl_xml_init(src_braille->device, src_braille->port, brl_xml_input_proc)) /* take Braille display from Gnopi... */
     {
 	brlval = FALSE;
-	fprintf(stderr, "\n** BRAILLE initialization failed for %s on port %d",
-					src_braille->device, src_braille->port);
+	fprintf(stderr, "\n** BRAILLE initialization failed for %s on port %s",
+					src_braille->device, src_braille->port ? src_braille->port : "<NONE>");
 	fprintf (stderr, "\n   To change one or more settings use gnopernicus UI\n");
     }
     else
     {
 	brlval = TRUE;
 	fprintf (stderr, "\n** BRAILLE initialization succeded for %s device"
-				    " on port %d.", src_braille->device, src_braille->port);
+				    " on port %s.", src_braille->device, src_braille->port ? src_braille->port : "<NONE>");
 	fprintf (stderr, "\n   To see something you must have a Braille device"
 				    " connected to your computer.");
 	fprintf (stderr, "\n   To change one or more settings use gnopernicus UI.\n");
@@ -878,7 +911,7 @@ src_braille_terminate ()
     brl_in_xml_terminate ();
 	
     g_free (src_braille->device);
-    src_braille->port = -1;
+    g_free (src_braille->port);
 }
 
 static SRCBraille* 
@@ -908,20 +941,47 @@ src_braille_set_device (gchar *device)
 }
 
 gboolean
-src_braille_set_port_no (gint port)
+src_braille_change_port (gchar *nport)
 {
     SRCBraille *braille;
-    
+
     braille = src_braille_get_global_settings ();
     sru_assert (braille);
-    
-    if (braille->port != port)
+
+    if (!nport || !braille->port || strcmp (braille->port, nport) != 0)
     {
-	braille->port = port;
+	g_free (braille->port);
+	braille->port = nport ? g_strdup (nport) : NULL;
 	src_use_braille = src_braille_restart ();
-	return TRUE;
-    }    
-    return FALSE;
+	return src_use_braille;
+    }
+
+    return TRUE;
+}
+
+
+gboolean
+src_braille_set_port ()
+{
+    gboolean rv;
+    gchar *nport = src_braille_get_port ();
+
+    rv = src_braille_change_port (nport);
+    g_free (nport);
+
+    return rv;
+}
+
+gboolean
+src_braille_set_port_no (gint port_no)
+{
+    gboolean rv;
+    gchar *nport = src_braille_get_port_from_no (port_no);
+
+    rv = src_braille_change_port (nport);
+    g_free (nport);
+
+    return rv;
 }
 
 gboolean
@@ -1073,17 +1133,6 @@ src_braille_get_position_sensor ()
     return braille->position_sensor;
 }
 
-gint
-src_braille_get_port_no ()
-{
-    SRCBraille *braille;
-    
-    braille = src_braille_get_global_settings ();
-    sru_assert (braille);
-    
-    return braille->port;
-}
-
 gchar*
 src_braille_get_translation_table ()
 {
